@@ -624,7 +624,7 @@ void ClipboardBrowser::preloadCurrentPage()
 {
     const int h = viewport()->contentsRect().height();
     const QModelIndex start = indexNear(0);
-    preload(h, false, start);
+    preload(h, 1, start);
 }
 
 void ClipboardBrowser::preloadCurrentPageLater()
@@ -633,7 +633,7 @@ void ClipboardBrowser::preloadCurrentPageLater()
         m_timerPreload.start();
 }
 
-void ClipboardBrowser::preload(int pixels, bool above, const QModelIndex &start)
+void ClipboardBrowser::preload(int pixels, int direction, const QModelIndex &start)
 {
     QElapsedTimer t;
     t.start();
@@ -641,15 +641,11 @@ void ClipboardBrowser::preload(int pixels, bool above, const QModelIndex &start)
     if ( m_timerUpdateSizes.isActive() )
         updateSizes();
 
-    const int s = spacing();
-    const int direction = above ? -1 : 1;
     int row = start.row();
     QModelIndex ind = start;
     int y = 0;
 
     const auto margins = m_sharedData->theme.margins();
-    const auto rowNumberSize = m_sharedData->theme.rowNumberSize();
-    const auto padding = QPoint(rowNumberSize.width() + margins.width() - s, margins.height());
 
     int items = 0;
     bool anyShown = false;
@@ -658,7 +654,11 @@ void ClipboardBrowser::preload(int pixels, bool above, const QModelIndex &start)
             continue;
 
         const auto rect = visualRect(ind);
-        const auto pos = rect.topLeft() + padding;
+        const auto rowNumberWidth = m_sharedData->theme.rowNumberSize(row).width();
+        const auto pos = QPoint(
+            rect.left() + margins.width() - spacing() + rowNumberWidth,
+            rect.top() + margins.height()
+        );
 
         if ( d.showAt(ind, pos) )
             anyShown = true;
@@ -1061,26 +1061,36 @@ void ClipboardBrowser::dropEvent(QDropEvent *event)
 
 void ClipboardBrowser::paintEvent(QPaintEvent *e)
 {
-    // Hide items outside viewport.
-    const auto firstVisibleIndex = indexNear(0);
-    if ( firstVisibleIndex.isValid() ) {
-        for (int row = firstVisibleIndex.row() - 1; row >= 0; --row) {
-            auto w = d.cacheOrNull(row);
-            if (w)
-                w->widget()->hide();
+    const QRect rect = e->rect();
+    const int top = rect.top();
+    const int bottom = rect.bottom();
+
+    // Hide items outside viewport after scrolling.
+    if (top == 0) {
+        const int outside = viewport()->contentsRect().width() * 2;
+
+        const auto firstVisibleIndex = indexNear(top);
+        if ( firstVisibleIndex.isValid() ) {
+            for (int row = firstVisibleIndex.row() - 1; row >= 0; --row) {
+                auto w = d.cacheOrNull(row);
+                if (w)
+                    w->widget()->move(outside, 0);
+            }
         }
-    }
-    const int h = viewport()->contentsRect().height();
-    const auto lastVisibleIndex = indexNear(h - 3 * spacing());
-    if ( lastVisibleIndex.isValid() ) {
-        for (int row = lastVisibleIndex.row() + 1; row < m.rowCount(); ++row) {
-            auto w = d.cacheOrNull(row);
-            if (w)
-                w->widget()->hide();
+
+        const auto lastVisibleIndex = indexNear(bottom);
+        if ( lastVisibleIndex.isValid() ) {
+            for (int row = lastVisibleIndex.row() + 1; row < m.rowCount(); ++row) {
+                auto w = d.cacheOrNull(row);
+                if (w)
+                    w->widget()->move(outside, 0);
+            }
         }
     }
 
-    preloadCurrentPage();
+    // Load items inside the current paint rectangle.
+    const QModelIndex start = indexNear(top);
+    preload(bottom, 1, start);
 
     QListView::paintEvent(e);
 
@@ -1439,13 +1449,11 @@ void ClipboardBrowser::keyPressEvent(QKeyEvent *event)
         // Preload next and previous pages so that up/down and page up/down keys scroll correctly.
         if ( !m_timerPreload.isActive() ) {
             if (key == Qt::Key_PageDown || key == Qt::Key_PageUp)
-                preload(h, (key == Qt::Key_PageUp), current);
+                preload(h, (key == Qt::Key_PageUp) ? -1 : 1, current);
             else if (key == Qt::Key_Down || key == Qt::Key_Up)
-                preload(2 * spacing(), (key == Qt::Key_Up), current);
+                preload(2 * spacing(), (key == Qt::Key_Up) ? -1 : 1, current);
             else if (key == Qt::Key_End)
-                preload(h, true, index(length() - 1));
-            scheduleDelayedItemsLayout();
-            executeDelayedItemsLayout();
+                preload(h, -1, index(length() - 1));
         }
 
         if (key == Qt::Key_PageDown || key == Qt::Key_PageUp) {
