@@ -233,12 +233,6 @@ QString getFontStyleSheet(const QString &fontString, double scale = 1.0)
     return result;
 }
 
-int itemMargin()
-{
-    const int dpi = QGuiApplication::primaryScreen()->physicalDotsPerInchX();
-    return std::max(2, dpi / 72);
-}
-
 QString themePrefix()
 {
 #ifdef COPYQ_THEME_PREFIX
@@ -545,8 +539,7 @@ void Theme::resetTheme()
 
 void Theme::updateTheme()
 {
-    const auto margin = itemMargin();
-    m_margins = QSize(margin + 2, margin);
+    m_margins = QSize(2, 2);
 
     // search style
     m_searchPalette.setColor(QPalette::Base, color("find_bg"));
@@ -562,12 +555,19 @@ void Theme::updateTheme()
     m_showRowNumber = value("show_number").toBool();
     m_rowNumberPalette.setColor(QPalette::Text, color("num_fg"));
     m_rowNumberFont = font("num_font");
-    m_rowNumberSize = QFontMetrics(m_rowNumberFont).boundingRect( QLatin1String("0123") ).size()
-            + QSize(m_margins.width(), m_margins.height());
+    m_rowNumberFontMetrics = QFontMetrics(m_rowNumberFont);
 
     m_antialiasing = value("font_antialiasing").toBool();
 }
 
+QSize Theme::rowNumberSize(int n) const
+{
+    if (!m_showRowNumber)
+        return QSize(0, 0);
+
+    return m_rowNumberFontMetrics.boundingRect(QString::number(n + m_rowIndexFromOne)).size() + m_margins;
+}
+
 void Theme::decorateBrowser(QAbstractScrollArea *c) const
 {
     decorateScrollArea(c);
